@@ -37,10 +37,10 @@ MODULE_POSTFIX="_"
 EXEC_TYPES_DRBG10X__64_bit___="CFLAGS=-DOPENSSL_DRBG_10X"
 EXEC_TYPES_DRBG10X__32_bit___="CFLAGS=\"-m32 -DOPENSSL_DRBG_10X\" LDFLAGS=-m32"
 
-EXEC="TDES_C FFC_DH KBKDF"
+EXEC="TDES_C FFC_DH KBKDF TLS_v1.3"
 
 # Environment variable evaluated by ACVP Parser
-OPENSSL_REMOVE_FFC_DH="OPENSSL_ACVP_DH_KEYGEN=1"
+CIPHER_CALL_FFC_DH="OPENSSL_ACVP_DH_KEYGEN=1"
 
 if [ $(uname -m) = "s390x" ]; then
 	EXEC="$EXEC
@@ -59,21 +59,21 @@ elif [ $(uname -m) = "aarch64" ]; then
 	# define ARMV8_SHA256    (1<<4)
 	# define ARMV8_PMULL     (1<<5)
 	# define ARMV8_SHA512    (1<<6)
-	OPENSSL_REMOVE_TDES_C="OPENSSL_armcap_P=0"
-	OPENSSL_REMOVE_AES_C_GCM="OPENSSL_armcap_P=0"
-	OPENSSL_REMOVE_AES_C="OPENSSL_armcap_P=0"
-	OPENSSL_REMOVE_SHA_ASM="OPENSSL_armcap_P=0"
-	OPENSSL_REMOVE_SHA3_ASM="OPENSSL_armcap_P=0"
+	CIPHER_CALL_TDES_C="OPENSSL_armcap_P=0"
+	CIPHER_CALL_AES_C_GCM="OPENSSL_armcap_P=0"
+	CIPHER_CALL_AES_C="OPENSSL_armcap_P=0"
+	CIPHER_CALL_SHA_ASM="OPENSSL_armcap_P=0"
+	CIPHER_CALL_SHA3_ASM="OPENSSL_armcap_P=0"
 
 	# Set the NEON bit
-	OPENSSL_REMOVE_VPAES="OPENSSL_armcap_P=1"
-	OPENSSL_REMOVE_SHA_NEON="OPENSSL_armcap_P=1"
+	CIPHER_CALL_VPAES="OPENSSL_armcap_P=1"
+	CIPHER_CALL_SHA_NEON="OPENSSL_armcap_P=1"
 
 	# Set the CE bits
-	OPENSSL_REMOVE_CE="OPENSSL_armcap_P=7D"
-	OPENSSL_REMOVE_CE_GCM="OPENSSL_armcap_P=7D"
-	OPENSSL_REMOVE_SHA_CE="OPENSSL_armcap_P=7D"
-	OPENSSL_REMOVE_SHA3_CE="OPENSSL_armcap_P=7D"
+	CIPHER_CALL_CE="OPENSSL_armcap_P=7D"
+	CIPHER_CALL_CE_GCM="OPENSSL_armcap_P=7D"
+	CIPHER_CALL_SHA_CE="OPENSSL_armcap_P=7D"
+	CIPHER_CALL_SHA3_CE="OPENSSL_armcap_P=7D"
 else
 	EXEC="$EXEC
 	      AESNI AESNI_AVX AESNI_CLMULNI AESNI_ASM
@@ -134,7 +134,7 @@ do_test() {
 		eval "$BUILD_FLAGS build_tool ${TARGET}"
 
 		for exec in $EXEC; do
-			eval CIPHER_CALL=\$CIPHER_CALL_$exec
+			eval CIPHER_CALL=\${CIPHER_CALL_${exec}} 2> /dev/null
 
 			local modulename="${MODULE_PREFIX}${type}${exec}${MODULE_POSTFIX}"
 
