@@ -45,6 +45,7 @@
 #include "parser_pbkdf.h"
 #include "parser_hkdf.h"
 #include "parser_ifc.h"
+#include "parser_tls13.h"
 
 #ifdef __cplusplus
 extern "C"
@@ -226,6 +227,7 @@ DEF_CALLBACK_TYPE(kdf_108)
 DEF_CALLBACK_TYPE(pbkdf)
 DEF_CALLBACK_TYPE(hkdf)
 DEF_CALLBACK_TYPE(kts_ifc)
+DEF_CALLBACK_TYPE(tls13)
 
 /**
  * @brief json_callback specifies one generic callback. It therefore wraps the
@@ -280,6 +282,7 @@ enum {
 	CB_TYPE_pbkdf,
 	CB_TYPE_hkdf,
 	CB_TYPE_kts_ifc,
+	CB_TYPE_tls13,
 };
 struct json_callback {
 	union {
@@ -323,6 +326,7 @@ struct json_callback {
 		struct pbkdf_callback pbkdf;
 		struct hkdf_callback hkdf;
 		struct kts_ifc_callback kts_ifc;
+		struct tls13_callback tls13;
 	} callback;
 	uint32_t cb_type;
 	flags_t flags;
